/*******************************************************************
* NAME
*   @file   drvPage/IMMDCompress.h
* SYNOPSIS
*   Compresses and decompresses 8-bit color data. This is opened for publishing
* DESCRIPTION
*   
* NOTES
* HISTORY
*   Rev.    Date        BY
*   01e 2001/06/11  Hirasawa(IP)   Added conversion function for X data uncompressing format
*   01d 2001/06/05  Hirasawa(IP)   Added limit value of data after compression in raster units and deleted SFDRephufCompress()
*   01c 2001/04/13  Hirasawa(IP)    Added Gray -> K conversion function
*   01b 2000/12/14  Hirasawa(IP)   Changed STATUS_ERROR,STATUS_OK,UINT32,SINT32
*   01a 2000/12/07  Hirasawa(IP)    1st.
* COPYRIGHT
*   OP DESIGN GRP   SEIKO EPSON CORP.
*******************************************************************/

#ifndef   IMMDCOMPRESS_H
#define   IMMDCOMPRESS_H
#ifdef __cplusplus
extern "C" {
#endif

/* ---------------- Type ----------------*/
#include "epson-typedefs.h"

#if 0//iPrint
/* return boolean values */
#if         !defined(BOOL)
    typedef  int     BOOL;
#endif
#if !defined(TRUE)  || (TRUE  !=1)
    #define  TRUE       1
#endif
#if !defined(FALSE) || (FALSE !=0)
    #define  FALSE      0
#endif
#endif//iPrint

/* return status values */
#if         !defined(STATUS)
    typedef  int     STATUS;
#endif
#if !defined(STATUS_OK)    || (STATUS_OK    !=0)    /* 01b old OK */
    #define  STATUS_OK            0
#endif
#if !defined(STATUS_ERROR) || (STATUS_ERROR !=-1)   /* 01b old ERROR */
    #define  STATUS_ERROR       (-1)
#endif

#if                 !defined(Byte)      /*  8 bit */
    typedef  unsigned char   Byte;
#endif
#if                 !defined(HWord)     /* 16 bit */
    typedef  unsigned short  HWord;
#endif
#if                 !defined(Word)      /* 32 bit */
    typedef  unsigned long   Word;
#endif
#if                 !defined(UCHAR)     /*  8 bit */
    typedef  unsigned char   UCHAR;
#endif
#if                 !defined(USHORT)    /* 16 bit */
    typedef  unsigned short  USHORT;
#endif
#if                 !defined(UINT)      /* 32 bit */
    typedef  unsigned int    UINT;
#endif
#if                 !defined(ULONG)     /* 32 bit */
    typedef  unsigned long   ULONG;
#endif
#if                 !defined(UINT8)
    typedef unsigned char    UINT8;
#endif
#if                 !defined(SINT8)
    typedef   signed char    SINT8;
#endif
#if                 !defined(UINT16)
    typedef unsigned short   UINT16;
#endif
#if                 !defined(SINT16)
    typedef   signed short   SINT16;
#endif
#if                 !defined(UINT32)
    typedef unsigned int     UINT32;    /* 01b old long */
#endif
#if                 !defined(SINT32)
    typedef   signed int     SINT32;    /* 01b old long */
#endif


/* ---------------- Constant ----------------*/
#define SIZE_OF_RGBX		(4)			//*< RGBX^f[^̃TCY
#if defined(CM_LITTLE_ENDIAN)
	#define OFFSET_TO_R		(0)			//*< RGBX^f[^ɂR(Red)vfւ̃ItZbg
	#define OFFSET_TO_X		(3)			//*< RGBX^f[^ɂX(Attribute)vfւ̃ItZbg
#else
	#define OFFSET_TO_R		(3)			//*< RGBX^f[^ɂR(Red)vfւ̃ItZbg
	#define OFFSET_TO_X		(0)			//*< RGBX^f[^ɂX(Attribute)vfւ̃ItZbg
#endif

/* ---------------- Structure ----------------*/


/* ---------------- Global variable definition ----------------*/


/* ---------------- Macro ----------------*/

#if 0//iPrint
/* ---------------- Publish function definition ----------------*/
long CompressPlane(       Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int  nbit, short cmp_limit, BOOL X_data); /* 01e */
long CompressPlaneRev(    Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int  nbit, short cmp_limit, BOOL X_data); /* 01e */
long DecompressPlane(     Byte *src_buf, Word src_w, Word src_h, Byte *dec_buf,                Byte *tmp_buf, Byte *rbt_buf, int  nbit);
long CompressPlaneGray(   Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int  nbit); /* 01c */
long CompressPlaneGrayRev(Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int  nbit); /* 01c */
long DecompressPlaneGray( Byte *src_buf, Word src_w, Word src_h, Byte *dec_buf,                Byte *tmp_buf, Byte *rbt_buf, int  nbit); /* 01c */
long CompressXMbit(       Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size,                Byte *rbt_buf, Byte mbit); /* 01e */
long CompressXMbitRev(    Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size,                Byte *rbt_buf, Byte mbit); /* 01e */
#endif//iPrint
EPS_INT32 RasterByteSizeTableCreate(EPS_UINT32 src_w, EPS_UINT32 src_h, EPS_UINT8 *rbt_buf);

#if 0//iPrint
long CompressPlane_pen4(       Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int  nbit, short cmp_limit, BOOL X_data); /* 01e */
long CompressPlaneRev_pen4(    Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int  nbit, short cmp_limit, BOOL X_data); /* 01e */
long CompressPlaneGray_pen4(   Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int  nbit); /* 01c */
long CompressPlaneGrayRev_pen4(Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int  nbit); /* 01c */
#endif//iPrint
EPS_INT32 CompressPlaneREP(EPS_UINT8 *src_buf, EPS_UINT32 src_w, EPS_UINT32 src_h, EPS_UINT8 *cmp_buf, EPS_UINT32 cmp_size, EPS_UINT8 *tmp_buf, EPS_UINT8 *rbt_buf, EPS_INT32 nbit, EPS_INT16 cmp_limit, EPS_BOOL X_data);
#if 0//iPrint
long CompressPlaneREPRev(Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int nbit, short cmp_limit, BOOL X_data);

long CompressPlaneREP_pen4(Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int nbit, short cmp_limit, BOOL X_data);
long CompressPlaneREPRev_pen4(Byte *src_buf, Word src_w, Word src_h, Byte *cmp_buf, Word cmp_size, Byte *tmp_buf, Byte *rbt_buf, int nbit, short cmp_limit, BOOL X_data);
#endif//iPrint

#ifdef __cplusplus
}
#endif
#endif /* IMMDCOMPRESS_H */

/* end of IMMDCompress.h */
