/*******************************************************************
* NAME
*   @file   drvPage/IMMDCompresslc.h
* SYNOPSIS
*   Compresses and decompresses 8-bit color data. This is for relation definition local.
* DESCRIPTION
*   Including test program definition
* NOTES
* HISTORY
*   Rev.    Date        BY
*   01d 2001/04/13  Hirasawa(IP)    Deleted the macro for debugging
*   01c 2001/02/27  Hirasawa(IP)    Speeded up, added the ABS macro, included string.h, and defined NULL
*   01b 2000/12/15  Hirasawa(IP)   Added N-bit uncompressing mode C_NBIT_UNCOMP and CMYK_NBIT_X_1BIT_UNCOMP
*   01a 2000/12/07  Hirasawa(IP)    1st.
* COPYRIGHT
*   OP DESIGN GRP   SEIKO EPSON CORP.
*******************************************************************/

#ifndef   IMMDCOMPRESSLC_H
#define   IMMDCOMPRESSLC_H

/* -------- Inclusion of the header file --------*/
#include "string.h"
#include "IMMDCompress.h"

/* ---------------- Constant ----------------*/

#ifndef NULL
#ifdef __cplusplus
#define NULL    0
#else
#define NULL    ((void *)0)
#endif
#endif

/*  #define REHUF_MAX_COLOR          5      */  /* CMYKX */
/*  #define RASTER_ADRS_TBL_SIZE    (4 + 4) */  /* offset + byte */
/*  #define REHUF_WORK_SIZE_MUL     (4 * 2) */  /* * 4 = long, * 2 = 2 raster */
/*  #define BOUNDARY                256     */  /* malloc bounsary adjust */
/*  #define FILE_NAME_LEN           256     */  /* file name length */

/**************************************************************************
*   Compression parameter initial value
***************************************************************************/
#define NUM1_INIT       1           /* Number of matches last but one */
#define NUM0_INIT       1           /* Number of last matches */
#define PRE1_INIT       0xFF        /* Previous pixel value */
#define PRE2_INIT       0x00        /* Pixel value last but one */
#define SW_INIT         2           /* Previous compression pattern */

/**************************************************************************
*   Flag code definition
**************************************************************************/
#define FLAG0_SW2_CODE      0x00
#define FLAG2_SW2_CODE      0x01
#define FLAG1_SW2_CODE      0x03
#define FLAG31_SW2_CODE     0x07
#define FLAG3_SW2_CODE      0x0F
#define FLAG3_SW3_CODE      0x00
#define FLAG31_SW34_CODE    0x01
#define FLAG0_SW34_CODE     0x03
#define FLAG1_SW34_CODE     0x07
#define FLAG2_SW34_CODE     0x0F
#define FLAG4_SW4_CODE      0x00
#define FLAG3_SW4_CODE      0x1F
#define FLAG6_SW234_CODE    0x7F

/**************************************************************************
*   Flag code length definition
**************************************************************************/
#define FLAG0_SW2_LENG      1
#define FLAG2_SW2_LENG      2
#define FLAG1_SW2_LENG      3
#define FLAG31_SW2_LENG     4
#define FLAG3_SW2_LENG      5
#define FLAG3_SW3_LENG      1
#define FLAG31_SW34_LENG    2
#define FLAG0_SW34_LENG     3
#define FLAG1_SW34_LENG     4
#define FLAG2_SW34_LENG     5
#define FLAG4_SW4_LENG      1
#define FLAG3_SW4_LENG      6
#define FLAG6_SW234_LENG    8

/**************************************************************************
*   huf code definition
**************************************************************************/
#define HUF0_CODE       0x05
#define HUF1_CODE       0x00
#define HUF2_CODE       0x02
#define HUF3_CODE       0x01
#define HUF4_CODE       0x03
#define HUF5_CODE       0x07
#define HUF6_CODE       0x0f
#define HUF7_CODE       0x1f
#define HUF8_CODE       0x3f
#define HUFEND_CODE     0x7f

/**************************************************************************
*   huf code length definition
**************************************************************************/
#define HUF0_LENG       3
#define HUF1_LENG       2
#define HUF2_LENG       2
#define HUF3_LENG       3
#define HUF4_LENG       3
#define HUF5_LENG       4
#define HUF6_LENG       5
#define HUF7_LENG       6
#define HUF8_LENG       7
#define HUFEND_LENG     7

/**************************************************************************
*   Compression mode
**************************************************************************/
typedef enum {
    RGBX_8BIT_COMP = 0,         /* Each RGBX 8-bit compression (without mask of X) */
    R_G_8BIT_COMP,              /* Each (R-G)G(B-G)X 8-bit compression (without mask of X) */
    YUVX_8BIT_COMP,             /* Each YUVX 8-bit compression (without mask of X) */
    RGBX_8BIT_UNCOMP,           /* Converts each RGBX to 8-bit uncompressing printable band (not support VIP2/VIPS ) */
    C_8BIT_COMP,                /* C(MYK) single plane 8-bit compression */
    C_NBIT_COMP,                /* C(MYK) single plane N-bit compression (shifts MSB N[1 to 7]bit to LSB) */
    C_8BIT_UNCOMP,              /* Converts C(MYK) single plane to 8-bit uncompressing printable band */
    C_NBIT_UNCOMP,              /* C(MYK) single plane N-bit uncompressing (shifts MSB N[1 to 7]bit to LSB) 01b */
    X_8BIT_COMP,                /* X single plane 8-bit compression (masks other than LSB with 0 and then compresses them) */
    X_1BIT_UNCOMP,              /* Converts X single plane to 1-bit uncompressing printable band (deletes other than LSB) */
    CMYK_8BIT_X_8BIT_COMP,      /* Each CMYK 8-bit + X 8-bit compression (masks other than LSB with 0 and then compresses them) */
    CMYK_NBIT_X_8BIT_COMP,      /* Each CMYK N-bit + X 8-bit compression (masks other than LSB with 0 and then compresses them) */
    CMYK_8BIT_X_1BIT_UNCOMP,    /* Converts each CMYK 8-bit + X 1-bit to uncompressing printable band */
    CMYK_NBIT_X_1BIT_UNCOMP     /* Each C(MYK) N-bit + X 1-bit uncompressing (shifts MSB N[1 to 7]bit to LSB) 01b */
}   Vip2Rhv2CompMode;

/**************************************************************************
*   Raster boundary of data before and after compression
**************************************************************************/
typedef enum {
    BYTE_1_BOUND = 0,               /* 1-byte boundary */
    BYTE_8_BOUND                    /* 8-byte boundary */
}   Rhv2Boundary;

typedef struct {
    Rhv2Boundary    orgRasBound;    /* Raster boundary before compression */
    Rhv2Boundary    compRasBound;   /* Raster boundary after compression */
}   Rhv2RasBound;

/**************************************************************************
*   Whether to refer to the previous raster
*       When compression mode=UNCOMP, it does not care.
**************************************************************************/
typedef enum {
    PRERAS_NOTUSE = 0,              /* Without previous raster reference */
    PRERAS_USE                      /* With previous raster reference */
}   Rhv2PreRasSel;

/**************************************************************************
*   Uncompressing operation in raster units existence
*       When compression mode=UNCOMP, it does not care.
**************************************************************************/
typedef enum {
    FLAG6_NOTUSE = 0,               /* Without uncompressing operation in raster units */
    FLAG6_USE                       /* With uncompressing operation in raster units */
}   Rhv2Flag6Sel;

/**************************************************************************
*   Compression limit unit selection and compression limit value
**************************************************************************/
typedef enum {
    RASTER_LIMIT_OVER = 0,          /* Limits in raster units */
    BAND_LIMIT_OVER,                /* Limits in band units */
    NOT_FAIL                        /* No limit (compression does not fail) */
}   Rhv2CompFail;

typedef struct {
    EPS_UINT32      rgbcmykRasLimit;    /* RGB/CMYK raster limit (available when it is RASTER_LIMIT_OVER) */
    EPS_UINT32      xRasLimit;          /* X raster limit (available when it is RASTER_LIMIT_OVER) */
}   Rhv2RasLimit;

typedef union {
    Rhv2RasLimit    rasterLimit;    /* Raster limit (available when it is RASTER_LIMIT_OVER) */
    EPS_UINT32      bandLimit;      /* Band limit (available when it is BAND_LIMIT_OVER) */
}   Rhv2LimitSize;

typedef struct {
    Rhv2CompFail    compFail;       /* Selects compression limit unit */
    Rhv2LimitSize   limitSize;      /* Compression limit value */
}   Rhv2CompLimit;

/**************************************************************************
*   Compression information structure
**************************************************************************/
typedef struct {
    EPS_UINT32          srcWidth;   /* Number of pixels of one raster in band (band width) */
    EPS_UINT32          srcHeight;  /* Number of band rasters (band height) */
    Vip2Rhv2CompMode    compMode;   /* Selects the compression mode */
    EPS_UINT8           nBit;       /* Data depth at N-bit compression/uncompressing  (N=1 to 7) 01b */
    Rhv2RasBound        rasBound;   /* Selects raster boundary of data before and after compression */
    Rhv2PreRasSel       prerasSel;  /* Selects whether to refer to the previous raster */
    Rhv2Flag6Sel        flag6Sel;   /* Selects whether to use the uncompressing operation in raster units (not needed for decompression) */
    Rhv2CompLimit       compLimit;  /* Selects compression limit unit and sets upper limit value of data after compression [byte] (not needed for decompression) */
}   Vip2Rhv2CompInfo;

/* ---------------- Structure ----------------*/


/* ---------------- Global variable definition ----------------*/


/* ---------------- Macro ----------------*/

/* Converts to data after decompression of succeeded N-bit compression
*   Source is data that is already shifted.
* [Target model]
*   Fantail or later models: Newly created (ASIC=VIP2, VIPS)
* @param    int   nbit     :N bit compress   Each color nbit compression (1 to 7 bit) (Number of bits to be enabled)
* @param    UINT8 cc      : source              Source (shifted data)
* @param    UINT8 wr      : destination         Writing destination
* @return   UINT8 wr      : Decompressed data
*/
    #define IMMD_COMP_NBIT_RESTORE(nbit, cc, wr)        \
    {   \
        EPS_UINT8 nTmpVal = cc; \
        switch (nbit) {                                 \
        case 1: nTmpVal = (UINT8)(cc<<7 | cc<<6 | cc<<5 | cc<<4 | cc<<3 | cc<<2 | cc<<1 | cc );  break;  \
        case 2: nTmpVal = (UINT8)(cc<<6 | cc<<4 | cc<<2 | cc );  break;  \
        case 3: nTmpVal = (UINT8)(cc<<5 | cc<<2 | cc>>1 );       break;  \
        case 4: nTmpVal = (UINT8)(cc<<4 | cc );                  break;  \
        case 5: nTmpVal = (UINT8)(cc<<3 | cc>>2 );               break;  \
        case 6: nTmpVal = (UINT8)(cc<<2 | cc>>4 );               break;  \
        case 7: nTmpVal = (UINT8)(cc<<1 | cc>>6 );               break;  \
        default:nTmpVal =         cc ;                           break;  \
        }   \
        wr = nTmpVal; \
    }

/* Computes the absolute value
* [Target model]
*   Fantail or later models: Newly created (ASIC=VIP2, VIPS) 01c
* @param    int  v : Input
* @return   int    : Absolute value of argument
*/
    #define  IMMD_COMP_ABS(v)  ((v) < 0 ? -(v) : (v))


/* ---------------- Publish function definition ----------------*/

#if 0 //iPrint
STATUS Vip2Rhv2CompressPlaneGray8bit_pen4(UINT8 *srcAdr, UINT8 *dstAdr, UINT8 *ratAdr, UINT8 *compWork, Vip2Rhv2CompInfo *compInfo, UINT32 *compSize);
STATUS Vip2Rhv2CompressPlaneGray8bitRev_pen4(UINT8 *srcAdr, UINT8 *dstAdr, UINT8 *ratAdr, UINT8 *compWork, Vip2Rhv2CompInfo *compInfo, UINT32 *compSize);
STATUS Vip2Rhv2CompressPlaneGrayNbit_pen4(UINT8 *srcAdr, UINT8 *dstAdr, UINT8 *ratAdr, UINT8 *compWork, Vip2Rhv2CompInfo *compInfo, UINT32 *compSize);
STATUS Vip2Rhv2CompressPlaneGrayNbitRev_pen4(UINT8 *srcAdr, UINT8 *dstAdr, UINT8 *ratAdr, UINT8 *compWork, Vip2Rhv2CompInfo *compInfo, UINT32 *compSize);
STATUS Vip2Rhv2CompressPlane8bit_pen4(UINT8 *srcAdr, UINT8 *dstAdr, UINT8 *ratAdr, UINT8 *compWork, Vip2Rhv2CompInfo *compInfo, UINT32 *compSize);
STATUS Vip2Rhv2CompressPlane8bitRev_pen4(UINT8 *srcAdr, UINT8 *dstAdr, UINT8 *ratAdr, UINT8 *compWork, Vip2Rhv2CompInfo *compInfo, UINT32 *compSize);
STATUS Vip2Rhv2CompressPlaneNbit_pen4(UINT8 *srcAdr, UINT8 *dstAdr, UINT8 *ratAdr, UINT8 *compWork, Vip2Rhv2CompInfo *compInfo, UINT32 *compSize);
STATUS Vip2Rhv2CompressPlaneNbitRev_pen4(UINT8 *srcAdr, UINT8 *dstAdr, UINT8 *ratAdr, UINT8 *compWork, Vip2Rhv2CompInfo *compInfo, UINT32 *compSize);
#endif //iPrint

#endif /* IMMDCOMPRESSLC_H */

/* end of IMMDCompresslc.h */
