/*
 * EPSON ESC/P-R Printer Driver for Linux
 * Copyright (C) 2002-2005 AVASYS CORPORATION.
 * Copyright (C) Seiko Epson Corporation 2002-2005.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * As a special exception, AVASYS CORPORATION gives permission to
 * link the code of this program with libraries which are covered by
 * the AVASYS Public License and distribute their linked
 * combinations.  You must obey the GNU General Public License in all
 * respects for all of the code used other than the libraries which
 * are covered by AVASYS Public License.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <unistd.h>
#include <dlfcn.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include "err.h"
#include "mem.h"
#include "optBase.h"
#include "epson-escpr-def.h"
#include "epson-escpr-media.h"
//#include "epson-protocol.h"	// LPS7160
#include "epson-escpr-pvt.h"
#include "epson-escpr-api.h"
#include "epson-escpr-services.h"
#include "epson-escpr-mem.h"
#include "epson-escpage-s.h"
#include "linux_cmn.h"


#define PATH_MAX 256
#define NAME_MAX 41

#define	EPS_PARAM_MP_TRAY				0x00
#define	EPS_PARAM_CASSETTE1				0x01
#define	EPS_PARAM_CASSETTE2				0x02
#define	EPS_PARAM_CASSETTE3				0x03
#define	EPS_PARAM_CASSETTE4				0x04
#define	EPS_PARAM_MANUALFEED			0x0A
#define	EPS_PARAM_AUTO_TRAY				0xFF

#define WIDTH_BYTES(bits) (((bits) + 31) / 32 * 4)

#define PIPSLITE_FILTER_VERSION "* epson-escpr is a part of " PACKAGE_STRING

#define PIPSLITE_FILTER_USAGE "Usage: $ epson-escpr model width_pixel height_pixel Ink PageSize Quality"


#ifndef EPS_END_PAGE
#define EPS_END_PAGE                    0       /* There is no next page                */
#endif

#ifndef	EPS_NEXT_PAGE
#define EPS_NEXT_PAGE                   1       /* There is a next page                 */
#endif

/* static functions */
static int getMediaTypeID(char *);
static int getMediaSizeID(char *);
const int band_line = 1;

extern EPS_INT32    libStatus;                  /*  Library (epsInitDriver) status      */
extern EPS_PRINT_JOB   printJob;
extern EPS_UINT32   sendDataBufSize;
extern EPS_UINT8*   sendDataBuf;    /* buffer of SendCommand(save) input                */
extern EPS_UINT32   tmpLineBufSize;
extern EPS_UINT8*   tmpLineBuf;

extern EPS_CMN_FUNC epsCmnFnc;

//extern EPS_JOB_FUNCS jobFnc;	// LPS7160

extern EPS_INT32 tonerSave;
extern EPS_INT32 back_type;
extern EPS_INT32 lWidth;
extern EPS_INT32 lHeight;
extern EPS_INT32 areaWidth;
extern EPS_INT32 areaHeight;
//CMINFO gCMInfo;	// LPS7160
//void*   gpCMTD;	// LPS7160

#ifndef ESCPR_HEADER_LENGTH	
#define ESCPR_HEADER_LENGTH            10    /* ESC + CLASS + ParamLen + CmdName */	
#endif

#ifndef ESCPR_SEND_DATA_LENGTH
#define ESCPR_SEND_DATA_LENGTH          7
#endif


EPS_ERR_CODE epsInitLib(){
	EPS_CMN_FUNC cmnFuncPtrs;
	memset(&cmnFuncPtrs, 0, sizeof(EPS_CMN_FUNC));
	
	cmnFuncPtrs.version = EPS_CMNFUNC_VER_CUR;
	cmnFuncPtrs.findCallback = NULL;
	cmnFuncPtrs.memAlloc = &epsmpMemAlloc;
	cmnFuncPtrs.memFree = &epsmpMemFree;
	cmnFuncPtrs.sleep = &epsmpSleep;
	cmnFuncPtrs.getTime = &epsmpGetTime;
	cmnFuncPtrs.getLocalTime = &epsmpGetLocalTime;
	cmnFuncPtrs.lockSync = &epsmpLockSync;
	cmnFuncPtrs.unlockSync = &epsmpUnlockSync;
	cmnFuncPtrs.stateCallback = NULL; /* current version unused */
	memcpy((void*)(&epsCmnFnc), (void*)&cmnFuncPtrs, sizeof(EPS_CMN_FUNC));
}

EPS_ERR_CODE epsInitJob(){
	
	memset(&printJob, 0, sizeof(EPS_PRINT_JOB));
	printJob.printer = (EPS_PRINTER_INN*) malloc(sizeof(EPS_PRINTER_INN));
	memset(printJob.printer, 0, sizeof(EPS_PRINTER_INN));
	
	printJob.jobStatus  = EPS_STATUS_NOT_INITIALIZED;
	printJob.pageStatus = EPS_STATUS_NOT_INITIALIZED;
	printJob.findStatus = EPS_STATUS_NOT_INITIALIZED;
	printJob.bComm		= TRUE;	
	printJob.platform   = 0x04; /* '0x04 = linux' is default */ 

	libStatus   = EPS_STATUS_NOT_INITIALIZED;
	sendDataBufSize = 0;

	/* DEL printJob.additional = EPS_ADDDATA_NONE;
	printJob.qrcode.bits = NUL	sendDataBuf = NULL;
	tmpLineBufSize = 0;
	tmpLineBuf  = NULL;

/*** Initialize continue buffer                                                         */
	//printJob.contData.sendData = NULL;
	//printJob.contData.sendDataSize = 0;
	//printJob.qrcode.cellNum = 0;
	//printJob.qrcode.dpc = 0; */
	obsClear();

/*** Set "Endian-ness" for the current cpu                                              */
	memInspectEndian();
	
/*** Set Communication Mode                                                             */
	//printJob.commMode = commMode;
	
/*** Change ESC/P-R Lib Status                                                          */
	libStatus = EPS_STATUS_INITIALIZED;
	EPS_PRINTER_INN*    printer = printJob.printer;
//	printer->pmData.state = EPS_PM_STATE_NOT_FILTERED;	// LPS7160
	printer->language = EPS_LANG_ESCPAGE_S04;			// LPS7160
	return EPS_ERR_NONE;
}


EPS_ERR_CODE epsInitVariable(){
	sendDataBufSize = (EPS_INT32)(ESCPR_HEADER_LENGTH    +
									ESCPR_SEND_DATA_LENGTH +
									(printJob.printableAreaWidth * printJob.bpp));
	sendDataBuf = (EPS_UINT8*)EPS_ALLOC(sendDataBufSize);
	if(sendDataBuf == NULL){
		sendDataBufSize = 0;
		EPS_RETURN( EPS_ERR_MEMORY_ALLOCATION );
	}
	
	memset(sendDataBuf, 0xFF, (EPS_UINT32)sendDataBufSize);
	/*** Allocate buffer for RLE complession                                        */
	tmpLineBufSize = (EPS_INT32)(printJob.printableAreaWidth * printJob.bpp) + 256; /* 256 is temp buffer */
	tmpLineBuf = (EPS_UINT8*)EPS_ALLOC(tmpLineBufSize);
	if(tmpLineBuf == NULL){
		tmpLineBufSize = 0;
		EPS_RETURN( EPS_ERR_MEMORY_ALLOCATION );
	}
	memset(tmpLineBuf, 0xFF, (EPS_UINT32)tmpLineBufSize);
	return EPS_ERR_NONE;
}


EPS_ERR_CODE epsFilterEndPage(EPS_BOOL bNextPage){
	EPS_ERR_CODE    retStatus= EPS_ERR_NONE;
	
#ifdef GCOMSW_CMD_ESCPAGE_S
	debug_msg("GCOMSW_CMD_ESCPAGE defined\n");
		retStatus = pageEndPage();
#else
		retStatus = EPS_ERR_LANGUAGE_NOT_SUPPORTED;
#endif

	return retStatus;

}

void eps_toupper(char *str){
	int i = 0;
	int len;
	if(str == NULL || strlen(str) == 0){
		return;
	}
	len = strlen(str);
	for(i = 0; i < len; i++){
		str[i] = toupper(str[i]);
	}
	return;
}

int epsGetInputOption(const char** in_argv, const int in_argc, char *opt, char *dest){
	
	if(opt == NULL || strlen(opt) == 0 || dest == NULL ){
		return -1;
	}
	
	int i = 0;
	char *str = NULL;
	debug_msg("get option = %s\n", opt);
	for(i = 0; i < in_argc; i++){
		//debug_msg("i = %d, argv[%d] = %s\n", i, i, in_argv[i]);
		str = strstr(in_argv[i], opt);
		if(str != NULL && (str[strlen(opt)] == ':') ){
			debug_msg("get option success: %s\n", str);
			strcpy(dest, str + strlen(opt) + 1);
			//return str + strlen(opt) + 1;
			return 0;
		}
	}
	debug_msg("get option = %s ERROR\n", opt);
	return -1;
}

static int
get_option_for_arg (const char** argv, const int argc, EPS_JOB_ATTRIB *jobAttr)
{
	
	int error = 0;
	char buff[256];
	memset(buff, 0, sizeof(buff));

	error = epsGetInputOption (argv, argc, "Color", buff);
	if (error == 0) {
		eps_toupper(buff);
		debug_msg("Ink = %s\n", buff);
		
		if (strcmp (buff, "COLOR") == 0) {
 			jobAttr->colorMode =  EPS_CM_COLOR;
		}
		else{ 
			jobAttr->colorMode =  EPS_CM_MONOCHROME;
		}
   
	}else{
		debug_msg("%s\n", "Cannot get option: Color.");
		err_fatal ("Cannot get option: Color."); /* exit */
	}
	
	error = epsGetInputOption (argv, argc, "PageSize", buff);
	if (error == 0){
		debug_msg("PageSize = %s\n", buff);
		eps_toupper(buff);
		jobAttr->mediaSizeIdx = getMediaSizeID(buff);
		
	}else{
		debug_msg("%s\n", "Cannot get option: PageSize.");
		err_fatal ("Cannot get option: PageSize."); /* exit */
	}

	error = epsGetInputOption (argv, argc, "PaperType", buff);
	if (error == 0){
		eps_toupper(buff);
		debug_msg("PaperType = %s\n", buff);
		jobAttr->mediaTypeIdx = getMediaTypeID(buff);
		back_type = 0; 
		if (strstr(buff, "-BACK") != NULL)
		{
			back_type = 1;
			debug_msg("this type is back-side type\n");
		}
	}else{
		debug_msg("%s\n", "Cannot get option: PaperType.");
		err_fatal ("Cannot get option: PaperType."); /* exit */
	}
	
	error = epsGetInputOption (argv, argc, "Quality", buff);
	if (error == 0){
		debug_msg("Quality = %s\n", buff);
		if(strcmp(buff, "Quick") == 0){
			jobAttr->printQuality = EPS_MQID_DRAFT;
			jobAttr->inputResolution = EPS_IR_300X300;
			jobAttr->topMargin = 300/6;
		}else if(strcmp(buff, "Fine") == 0){
			jobAttr->printQuality = EPS_MQID_NORMAL;
			jobAttr->inputResolution = EPS_IR_600X600;
			jobAttr->topMargin = 600/6;
		}else{
			jobAttr->printQuality = EPS_MQID_DRAFT;
			jobAttr->inputResolution = EPS_IR_300X300;
			jobAttr->topMargin = 300/6;
		}
		
	}else{
		debug_msg("%s\n", "Cannot get option: Quality.");
		err_fatal ("Cannot get option: Quality."); /* exit */
	}

	error = epsGetInputOption (argv, argc, "Duplex", buff);
	if (error == 0){
		debug_msg("Duplex = %s\n", buff);
		if(strcmp(buff, "top") == 0){
			jobAttr->duplex = EPS_DUPLEX_SHORT;
		}else if(strcmp(buff, "left") == 0){
			jobAttr->duplex = EPS_DUPLEX_LONG;
		}else{
			jobAttr->duplex = EPS_DUPLEX_NONE;
		}
	}else{
		debug_msg("%s\n", "Cannot get option: Duplex.");
		err_fatal ("Cannot get option: Duplex."); /* exit */
	}
	
	

	//get paper source option
	error = epsGetInputOption (argv, argc, "InputSlot", buff);
	if (error == 0){
		debug_msg("Input slot = %s\n", buff);
		
		if(strcmp(buff, "AutoSelect") == 0){
			
			jobAttr->paperSource = EPS_MPID_AUTO;	// LPS7160
			
		}else if(strcmp(buff, "MPTray") == 0){
		
			jobAttr->paperSource = EPS_MPID_MPTRAY;	// LPS7160
			
		}else if(strcmp(buff, "Cassette1") == 0){
		
			jobAttr->paperSource = EPS_MPID_FRONT1;	// LPS7160
			
		}else if(strcmp(buff, "Cassette2") == 0){
		
			jobAttr->paperSource = EPS_MPID_FRONT2;	// LPS7160
			
		}else if(strcmp(buff, "Cassette3") == 0){
		
			jobAttr->paperSource = EPS_MPID_FRONT3;	// LPS7160
			
		}else if(strcmp(buff, "Cassette4") == 0){
		
			jobAttr->paperSource = EPS_MPID_FRONT4;	// LPS7160
			
		}else{
			jobAttr->paperSource = EPS_MPID_AUTO;	// LPS7160
		}
	}else{
		debug_msg("%s\n", "Cannot get option: InputSlot.");
		err_fatal ("Cannot get option: InputSlot."); /* exit */
	}
	
	//get tonersave option
	error = epsGetInputOption (argv, argc, "Tonersave", buff);
	if (error == 0){
		eps_toupper(buff);
		debug_msg("Tonersaver = %s\n", buff);
		if(strcmp(buff, "ON") == 0){
			tonerSave = 1; //ON
			debug_msg("tonerSave turn on\n");
		}else{
			tonerSave = 0; //OFF
			debug_msg("tonerSave turn off\n");
		}
	}else{
		debug_msg("%s\n", "Cannot get option: Tonersave.");
		err_fatal ("Cannot get option: Tonersave."); /* exit */
	}
	
	error = epsGetInputOption (argv, argc, "Copies", buff);
	if (error == 0){
		jobAttr->copies = atoi(buff);
		if(jobAttr->copies < 1 || jobAttr->copies > 999){
			jobAttr->copies = 1;
		}
	}else{
		debug_msg("%s\n", "Cannot get option: Copies.");
		err_fatal ("Cannot get option: Copies."); /* exit */
	}
	
	
	jobAttr->printLayout = EPS_MLID_BORDERS;
	jobAttr->feedDirection = EPS_FEEDDIR_PORTRAIT;      /* paper feed direction  hardcode */
	return 0;
}

int
main (int argc, char *argv[])
{

#ifndef EPS_FILTER
#define EPS_FILTER
#endif /* EPS_FILTER */
	int width_pixel, height_pixel;
	char cBuff[256];
	int pagestartpage = 0;
	int bytes_per_line;
	int byte_par_pixel;
	
	double print_area_x, print_area_y;
	unsigned char *image_raw;
	int leftMargin, topMargin;

	int read_size = 0;

	int err = 0;
	int i, j;
	EPS_INT32 heightPixels = 0; 

	

	/* 2004.04.15 added for 'cancel page' */
	int cancel = 0;
	
	EPS_JOB_ATTRIB     jobAttr;
	memset (&jobAttr, 0, sizeof(jobAttr));
	
	for(i = 0; i < argc; i++){
		debug_msg("%s ", argv[i]);
	}
	debug_msg("\n\n\n");
	
	if (get_option_for_arg ((const char**)argv, argc, &jobAttr))
	{
		fprintf (stderr, "Cannot read filter option. Cannot get option of PIPS.");
		debug_msga("Cannot read filter option. Cannot get option of PIPS.\n");
		return 1;
	}


	err = epsGetInputOption((const char**)argv, argc, "Width", cBuff);
	if(err != 0){
		debug_msg("%s\n", "Cannot get option: Width.");
		err_fatal ("Cannot get option: Width."); /* exit */
	}
	width_pixel = atoi(cBuff);
	
	err = epsGetInputOption((const char**)argv, argc, "Height", cBuff);
	if(err != 0){
		debug_msg("%s\n", "Cannot get option: Height.");
		err_fatal ("Cannot get option: Height."); /* exit */
	}
	height_pixel = atoi(cBuff);
	
	lWidth = width_pixel;
	lHeight = height_pixel;

	if (jobAttr.colorMode == EPS_CM_COLOR) {
 			byte_par_pixel = 3;
	}else{ 
			byte_par_pixel = 1;
	}

	epsInitLib();
	
	epsInitJob();
	
	err = SetupJobAttrib(&jobAttr);
	if (err){
		debug_msg("Error occurred in \"SetupJobAttrib\": %d\n", err);
		err_fatal ("Error occurred in \"SetupJobAttrib\".");	/* exit */
	}
	
	debug_msg("printJob.printableAreaWidth = %d, printJob.printableAreaHeight = %d, printJob.attr.inputResolution = %d\n", (int) printJob.printableAreaWidth, (int) printJob.printableAreaHeight, printJob.attr.inputResolution);
	
	printJob.paperWidth = width_pixel;
	printJob.paperHeight = height_pixel;

	epsInitVariable();

	switch(printJob.attr.inputResolution){
		case EPS_IR_300X300:
			printJob.topMargin      = 300/6;
   		printJob.leftMargin     = 300/6;
   		break;
   	case EPS_IR_600X600:
   		printJob.topMargin      = 600/6;
   		printJob.leftMargin     = 600/6;
   		break;
   	default:
   	printJob.topMargin      = 300/6;
   	printJob.leftMargin     = 300/6;
   		break;

	}
	
	leftMargin = printJob.leftMargin;
	topMargin = printJob.topMargin;
	
	printJob.printableAreaHeight  = (EPS_UINT32)lHeight - (printJob.topMargin + printJob.leftMargin);
    printJob.printableAreaWidth = (EPS_UINT32)lWidth - (printJob.topMargin + printJob.leftMargin);
	
	pageInitFunctions();	// LPS7160

	debug_msg("begin pageAllocBuffer()\n");
	err = pageAllocBuffer();
	if(err){
		debug_msg ("Error occurred in \"pageAllocBuffer\".");	/* exit */
		err_fatal ("Error occurred in \"pageAllocBuffer\".");	/* exit */
	}else{
		debug_msg ("pageAllocBuffer() success\n");
	}


//	EPS_PRINTER_INN curPrinter;					// LPS7160
//	memset(&curPrinter, 0, sizeof(curPrinter));	// LPS7160
//	curPrinter.language = EPS_LANG_ESCPAGE_S04;	// LPS7160
//	curPrinter.protocol = EPS_PROTOCOL_LPR;		// LPS7160

//	printJob.printer = &curPrinter;	// LPS7160

//	prtSetupJobFunctions(printJob.printer, &jobFnc);	// LPS7160

	err = pageStartJob();
	if (err){
		debug_msg("Error occurred in \"pageStartJob\": %d\n", err);
		err_fatal ("Error occurred in \"pageStartJob\".");	/* exit */
	}else{
		debug_msg("pageStartJob() success\n");
	}

//	err = pageSendTableColor();	// LPS7160
//	if (err){
//		debug_msg("Error occurred in \"PageSendTableColor\": %d\n", err);
//		err_fatal ("Error occurred in \"PageSendTableColor\".");	/* exit */
//	}else{
//		debug_msg("PageSendTableColor() success");
//	}

	print_area_x = printJob.printableAreaWidth;
	print_area_y = printJob.printableAreaHeight;
	
 	bytes_per_line = (width_pixel * byte_par_pixel);
	debug_msg("bytes_per_line = %d, byte_par_pixel = %d, width_pixel = %d\n", bytes_per_line, byte_par_pixel, width_pixel);
	
	int band_out_len = printJob.printableAreaWidth*byte_par_pixel;
	//band_out_len += band_out_len % 2;
	band_out_len = ((band_out_len +7)/8)*8;
	unsigned char *bandPrint = mem_new0(char, band_out_len*band_line);
	
	image_raw = mem_new0 (char, bytes_per_line);
	
	unsigned char *grayBuff;
	if(byte_par_pixel == 1){
		grayBuff = mem_new0 (char, band_out_len*band_line*3);
	}
	
	unsigned char *pPrint;
	
	debug_msg("\n\n\t\tDEBUG\n");
	debug_msg("paperWidth = %d\n", (int)printJob.paperWidth );
	debug_msg("paperLength = %d\n", (int)printJob.paperHeight );
	debug_msg("topMargin = %d\n", (int)printJob.topMargin );
	debug_msg("leftMargin = %d\n", (int)printJob.leftMargin );
	debug_msg("areaWidth = %d\n", (int)printJob.printableAreaWidth );
	debug_msg("areaLength = %d\n", (int)printJob.printableAreaHeight );
	debug_msg("band_out_len = %d\n", band_out_len );
	
	debug_msg("\n");
	
	int total_bytes = 0;
	read_size = 0;
	
	
	while(!cancel){
	debug_msg("filter step 1 \n");
	int y_count = 0;
	
		while(y_count < height_pixel && !cancel){
			debug_msg("while y_count \n");

			read_size = 0;
			while ((0 == err) && (read_size < bytes_per_line)) {
				debug_msg("while read_size \n");
				size_t rsize;
				rsize = read(STDIN_FILENO, image_raw + read_size, bytes_per_line - read_size);
				//debug_msg("rsize = %d\n", rsize);
				/* 2009.03.17 epson-escpr-1.0.0 */
				/* if user cancels job from CUPS Web Admin, */
				/* epson-escpr-wrapper exit normally, */
				/* and read() return rsize = 0 */
				debug_msg("rsize = %d\n", rsize);
				if (rsize <= 0) {
					if ((errno != EINTR) && (errno != EAGAIN) && (errno != EIO)) {
						err = -1;
						/* 2004.04.15		*/
						/* error then quit	*/ 
						/* don't care err = -1  */
						cancel = 1;
						/* not reached		*/
						break;
					}
					usleep(50000);
				} else {
					read_size += rsize;
					total_bytes += rsize;
					//debug_msg("read %d bytes, bytes_per_line = %d, y_count = %d\n", read_size, bytes_per_line, y_count);
				}
			}
			
			if(read_size == 0){
				//read data finish
				cancel = 1;
				goto FINISH;
			}
			debug_msg("step 100\n");
			if(y_count == 0 && !cancel){
				debug_msg("===> pageStartPage() call\n");
				err = pageStartPage();
				pagestartpage++;
				debug_msg("*********************%d*****************\n",pagestartpage);
				debug_msg("===> pageStartPage() finish\n");
				if (err){
				err_fatal ("Error occurred in \"PINIT_FUNC\"."); /* exit */
				}
				
				debug_msg("===> begin print, print_area_y = %d, height_pixel = %d\n", (int)print_area_y, (int) height_pixel);
				printJob.verticalOffset = 0;
				heightPixels = 0;
			}

// 			if(y_count > topMargin && (y_count-topMargin) % band_line == 0){
// 				debug_msg("<<<<set heightPixels = 0>>>\n");
// 				heightPixels = 0;
// 			}

			y_count ++;
			
			if(y_count <= topMargin || y_count > print_area_y + topMargin){
				//read all top, bottom margin data, don't print
				//debug_msg("<<<<y_count = %d>>>\n", y_count);
				continue;
			}

			memcpy (bandPrint + (heightPixels%band_line) * band_out_len, image_raw + leftMargin*byte_par_pixel , band_out_len );
			heightPixels ++;
			
			if( (heightPixels) == band_line || (y_count) == height_pixel){
				//debug_msg("BEGIN PrintBand, heightPixels = %d, y_count = %d\n", heightPixels, y_count);
				//heightPixels = 1;
				//band_line
				if(byte_par_pixel == 1){
					pPrint = grayBuff;
					int i = 0;
					for(i = 0; i < band_out_len*band_line; i++){
						grayBuff[i*3] = bandPrint[i];
						grayBuff[i*3+1] = bandPrint[i];
						grayBuff[i*3+2] = bandPrint[i];
					}
					err = PrintBand (grayBuff, band_out_len*band_line*3, &heightPixels);
				}else{
					err = PrintBand (bandPrint, band_out_len, &heightPixels);
				}
				heightPixels = 0;
				//err = PrintBand (image_raw, band_out_len, &heightPixels);
				if (err){
					if(err == EPS_OUT_OF_BOUNDS){
						debug_msg("EPS_OUT_OF_BOUNDS\n");
						break;
					}
					debug_msg("PrintBand error: %d\n", err);
					err_fatal ("Error occurred in \"OUT_FUNC\"."); /* exit */
				}else{
					debug_msg("PrintBand success\n");
				}
				//debug_msg("===> PrintBand() finish: heightPixels = %d, printJob.verticalOffset = %d\n", heightPixels, printJob.verticalOffset);
			}
	
		}
		debug_msg("pageEndPage() call\n", err);
		err = pageEndPage();
		if (err){
			debug_msg("pageEndPage() error: %d\n", err);
			err_fatal ("Error occurred in \"PEND_FUNC\".");	/* exit */
		}else{
			debug_msg("pageEndPage() success\n");
		}
FINISH:
		debug_msg("PRINT PAGE FINISH\n");
		
	}

	free(bandPrint);
	bandPrint = NULL;


	err = pageEndJob();
	if(err){
		debug_msg ("Error occurred in \"pageEndJob\"."); /* exit */
		err_fatal ("Error occurred in \"pageEndJob\"."); /* exit */
	}else{
		debug_msg("pageEndJob() success\n");
	}

	mem_free(image_raw);
	pageRelaseBuffer();
	debug_msg("print finish success, total_bytes = %ld\n", total_bytes);
	return 0;
}

static int  getMediaTypeID(char *rsc_name)
{
  int j;
	debug_msg("getMediaTypeID for [%s]\n", rsc_name);
  for(j = 0; mediaTypeData[j].value != END_ARRAY; j++)
    if(strncmp(mediaTypeData[j].rsc_name,rsc_name, strlen(mediaTypeData[j].rsc_name)) == 0){
    	debug_msg("mediaTypeData[j].rsc_name = %s, value = %d\n", mediaTypeData[j].rsc_name, mediaTypeData[j].value);
      return mediaTypeData[j].value;
    }
  return -1;
}

static int  getMediaSizeID(char *media_name){
  int j;

  for(j = 0; mediaSizeData[j].value != END_ARRAY; j++){
    if(strncmp(mediaSizeData[j].rsc_name,media_name, strlen(mediaSizeData[j].rsc_name)) == 0){
    	debug_msg("mediaSizeData[%d].rsc_name: %s, x_name = %s\n", j, mediaSizeData[j].rsc_name, mediaSizeData[j].x_name);
      return mediaSizeData[j].value;
    }
  }
  return -1;
}
