#!/bin/sh
# PostScript to Epson Laser Printer Filter script  Ver.2.0.0
# Copyright (C) Seiko Epson Corporation 2010-2015.
#
# This file is part of the "Epson Laser Printer Filter" program.
#
#
# The "Epson Laser Printer Filter" program is free software.  You can
# redistribute it and/or modify it under the terms of the AVASYS
# PUBLIC LICENSE (versions dated 2008-04-01 or later) as published by
# AVASYS CORPORATION.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of FITNESS
# FOR A PARTICULAR PURPOSE or MERCHANTABILITY.  See the AVASYS PUBLIC
# LICENSE for more details.
#
# You should have received a verbatim copy of the AVASYS PUBLIC
# LICENSE along with this program; if not, write to:
#
#       AVASYS CORPORATION.
#       1077-5, Otsu, Shimonogo, Ueda-shi,
#       Nagano-ken 386-1214 JAPAN
#

DEBUG="off"
PSOUT="off"
WORKF="off"

TOP=/tmp
#TOP=${TMPDIR:=/tmp}

ARGS="$@"

EPS_FILTER="/opt/epson-laser-printer-lp-s7160/cups/lib/filter/epson-escpage"


# for debug
#DEBUG="on"
#PSOUT="on"
#WORKF="on"

# get option value
optvalue()
{
    test -z "$1" && return 1
    KEY="$1"
    test -z "$(echo $ARGS | grep $KEY= )" && return 1
    VALUE=$(echo $ARGS | sed -e "s,^.*${KEY}=\\([^ ,\$]*\\).*,\\1,")
    return 0
}

# error message out
msgerr()
{
    cont=0 && test x"$1" = x"-n" && cont=1 && shift
    a="$@"
    test 0 -ne $# && echo -n "$@" >&2
    test 0 -eq $cont && echo "" >&2
    return 0
}

# Paper size in 600dpi
MODEL="" && optvalue Model && MODEL=${VALUE}
MODEL="lp-s7160"
test -z "${MODEL}" && msgerr "ERROR: $0: Un-known Peinter model name" && exit 1

TMPFILE="$TOP/pstolpf-${MODEL}"

WORKFLAG="${TMPFILE}_at_work_now"
clear_work_flag=""
if test "$WORKF" = "on" ; then
    clear_work_flag=" ; test -f $WORKFLAG && rm -f $WORKFLAG"
    while test -f $WORKFLAG ; do sleep 2 ; done
    sleep 2 && touch $WORKFLAG
fi

LOG=${TMPFILE}.log
test "$DEBUG" = "on" && echo $0 $ARGS > $LOG

tee0=""
#tee0="" && test "$DEBUG" = "on" && tee0="| tee ${TMPFILE}In.ps"
tee1="" && test "$DEBUG" = "on" && tee1="| tee ${TMPFILE}1.ps"
tee2="" && test "$DEBUG" = "on" && tee2="| tee ${TMPFILE}2.ps"
tee3="" && test "$DEBUG" = "on" && tee3="| tee ${TMPFILE}3.ps"
tee8="" && test "$DEBUG" = "on" && tee8="| tee ${TMPFILE}Out.ps"
tee9="" && test "$DEBUG" = "on" && tee9="| tee ${TMPFILE}.ras"
tee9=""
# log message out
msglog()
{
    test "$DEBUG" = "on" && echo "$@" >> $LOG
    return 0
}


#
# START MAIN
#

msglog "MODEL=${MODEL}"

# Paper size in 600dpi
XY600="" && optvalue XY600 && XY600=$VALUE && msglog "XY600=$XY600"
test -z "$XY600" && msgerr "ERROR: $0: Un-known paper size" && exit 1

# Output Quality.
Quality="" && optvalue Quality && Quality=$VALUE && msglog "Quality=$Quality"

if test x"$Quality" = x"Fine";then
	Resolution="600"
else
	Resolution="300"
fi	

# Output resolution.
#Resolution="" && optvalue Resolution && Resolution=$VALUE && msglog "Resolution=$Resolution"
test -z "$Resolution" && msgerr "ERROR: $0: Un-known output resolution" && exit 1
msglog "Resolution=$Resolution"
#Resolution=$(echo $Resolution | sed -e 's,^*[^[:digit:]],,;s,[^[:digit:]]*$,,')

# other options
PageSize=""  && optvalue PageSize  && PageSize=$VALUE  && msglog "PageSize=$PageSize"
Rotate=""    && optvalue Rotate    && Rotate=$VALUE    && msglog "Rotate=$Rotate"
Duplex=""    && optvalue Duplex    && Duplex=$VALUE    && msglog "Duplex=$Duplex"
Color=""     && optvalue Color   && Color=$VALUE     && msglog "Color=$Color"
MediaType="" && optvalue MediaType && MediaType=$VALUE && msglog "MediaType=$MediaType"
TonerSave="" && optvalue Tonersave && TonerSave=$VALUE && msglog "TonerSave=$TonerSave"
InputSlot="" && optvalue InputSlot && InputSlot=$VALUE && msglog "InputSlot=$InputSlot"
Copies=""    && optvalue Copies    && Copies=$VALUE    && msglog "Copies=$Copies"
Collate=""   && optvalue Collate   && Collate=$VALUE   && msglog "Collate=$Collate"
collatefile="-"
numcopies=1

# Output Printable area width/height in 600dpi.
LogX600="$(echo $XY600 | sed 's,x.*$,,')"
LogY600="$(echo $XY600 | sed 's,^.*x,,')"
msglog "LogX600=$LogX600" "LogY600=$LogY600"

# Logical Printable area width/height in PS unit (1/72 inch).
LogX72=$(echo "scale=6; a=$LogX600*72/600; scale=0; b=a/1; scale=6; c=a-b; if(c>0.444444) b+1 else b" | bc)
LogY72=$(echo "scale=6; a=$LogY600*72/600; scale=0; b=a/1; scale=6; c=a-b; if(c>0.444444) b+1 else b" | bc)
msglog "LogX72=$LogX72" "LogY72=$LogY72"

# if hardware rotate paper then swap X and Y.
if test x"$Rotate" = x"90";then
    X600=$LogY600
    Y600=$LogX600
    X72=$LogY72
    Y72=$LogX72
else
    X600=$LogX600
    Y600=$LogY600
    X72=$LogX72
    Y72=$LogY72
fi
msglog "X600=$X600" "Y600=$Y600"
msglog "X72=$X72" "Y72=$Y72"

# output printable area in output resolution unit(dot)
Xdot=$(echo "a=$X600*$Resolution/600; scale=0; b=a/1; scale=4; c=a-b; if(c>0.4444) b+1 else b" | bc)
Ydot=$(echo "a=$Y600*$Resolution/600; scale=0; b=a/1; scale=4; c=a-b; if(c>0.4444) b+1 else b" | bc)
msglog "Xdot=$Xdot" "Ydot=$Ydot"

# Duplex and rotation
useRotator="on"
if test x"$Rotate" = x"90";then
    # default duplex = long edge
    pstops1="-w$LogY72 -h$LogY72"
    if test x"$Duplex" = x"DuplexNoTumble"; then
	pstops2="2:R1\(0,$Y72\),R0\(0,$Y72\)"
    else
	if test x"$Duplex" = x"DuplexTumble"; then
	    pstops2="2:L1\($X72,0\),R0\(0,$Y72\)"
	else
	    pstops2="1:R0\(0,$Y72\)"
	fi
    fi
else
    # default duplex = short edge
    pstops1="-w$X72 -h$Y72"
    if test x"$Duplex" = x"DuplexNoTumble"; then
	pstops2="2:1U\($X72,$Y72\),0"
    else
	if test x"$Duplex" = x"DuplexTumble"; then
	    pstops2="2:1,0"
	else
	    pstops2="1:0"
#	    useRotator="off"
	fi
    fi
fi

# color/grayscale out control.
if test x"$Color" = x"MONO"; then
    gs1="-sDEVICE=bit -dGrayValues=256"
else
    gs1="-sDEVICE=bitrgb -dRedValues=256"
fi

gs2="-r$Resolution -g${Xdot}x${Ydot}"


# create ESC/PageS filter option argument.
opt="-o Width:${Xdot} -o Height:${Ydot}"
if test x"$Resolution" = x"300"; then
    opt="$opt -o Quality:Quick"
else
    opt="$opt -o Quality:Fine"
fi
if test x"$Duplex" = x"DuplexTumble"; then
    opt="$opt -o Duplex:top"
else
    if test x"$Duplex" = x"DuplexNoTumble"; then
    opt="$opt -o Duplex:left"
    else
    opt="$opt -o Duplex:Off"
    fi
fi

if test -n "$Color" ; then
    case "$Color" in
	"MONO")
	    opt="$opt -o Color:Black"
	    ;;
	"COLOR")
	    opt="$opt -o Color:Color"
	    ;;
	*)
	    ;;
    esac
fi

test -n "$PageSize" && opt="$opt -o PageSize:$PageSize"
test -n "$MediaType" && opt="$opt -o PaperType:$MediaType"
test -n "$InputSlot" && opt="$opt -o InputSlot:$InputSlot"
test -n "$TonerSave" && opt="$opt -o Tonersave:$TonerSave"
if test 1 -ge $((Copies)); then
    Copies="0"
    opt="$opt -o Copies:1"
else
    if test "$Collate" = "on"; then
	collatefile="$( mktemp $TOP/${MODEL}.XXXXXX )"
	cat > $collatefile
	numcopies=$((Copies))
	opt="$opt -o Copies:1"
    else
	test -n "$Copies" && opt="$opt -o Copies:$Copies"
    fi
fi

msglog "collatefile=$collatefile"
msglog "numcopies=$numcopies"

rotator="" && test "$useRotator" = "on" && rotator="| /usr/bin/pstops -q $pstops1 $pstops2 "
rasterizer="| gs -q -dBATCH -dPARANOIDSAFER -dNOPAUSE $gs1 $gs2 -sOutputFile=- -"
#escpagesfilter="| epson-lpf-${MODEL} $opt" && test "$PSOUT" = "on" && escpagesfilter=""
#escpagesfilter="| /usr/bin/lp-s7160 $opt" && test "$PSOUT" = "on" && escpagesfilter=""

escpagesfilter="| $EPS_FILTER $opt" && test "$PSOUT" = "on" && escpagesfilter=""


cmd="\
cat $collatefile \
$tee0 $rotator	  \
$tee8 $rasterizer \
$tee9 $escpagesfilter \
$clear_work_flag"

#msglog "cmd1 = $cmd"

#cmd="cat - | tee /tmp/pstolpf-lp-s7160In.ps"
# | /usr/bin/pstops -q -w842 -h842 1:R0\(0,595\) | tee /tmp/pstolpf-lp-s7160Out.ps"

#msglog $tee0 $rotator
#msglog $tee8 $rasterizer 
#msglog $tee9 $escpagesfilter

# execute transration.
page=1
while test 0 -ne $((numcopies));do
    msglog "page=$page numcopies=$numcopies"
    sh -c "$cmd"
    numcopies=$((numcopies -1))
    page=$((page +1))
done

test $collatefile != "-" && rm -f $collatefile

# for debug
msglog "[ Command ]"
msglog "$cmd"
msglog "command 2: $cmd2"
msglog "[ Emulation ]"
msglog "cat ${TMPFILE}In.ps $rotator > d1.ps"
msglog "cat d1.ps $rasterizer > d2.ras"
msglog "cat d2.ras $escpagesfilter > d3.prn"
msglog "[ View ]"
msglog "gs -q -r72 -g${LogX72}x${LogY72} -dBATCH -sDEVICE=x11 ${TMPFILE}In.ps"
msglog "gs -q -r72 -g${X72}x${Y72} -dBATCH -sDEVICE=x11 ${TMPFILE}Out.ps"
if test x"$Color" = x"black"; then
    TYPE="Gray"
else
    TYPE="RGB"
fi
msglog "convert -depth 8 -size ${Xdot}x${Ydot} $TYPE:${TMPFILE}.ras psto${MODEL}.png"
msglog "done"

exit 0
# end of script
